using Frontend.Assets;
using Microsoft.EntityFrameworkCore;

namespace Frontend.AssetStores;

public class PotatoStorage(DbContextOptions<PotatoStorage> options) : DbContext(options)
{
    public required DbSet<PotatoTransaction> Transactions { get; set; }

    public async Task AddPotatoes(PotatoTransaction transaction)
    {
        await Transactions.AddAsync(transaction);
        await SaveChangesAsync();
    }

    public async Task<PotatoTransaction> RemovePotatoes(double weight)
    {
        var transactions = Transactions
            .OrderBy(p => p.Id)
            .AsQueryable();

        var newTransaction = new PotatoTransaction();
        foreach (var transaction in transactions)
        {
            var newTransactionWeight = newTransaction.Weight + transaction.Weight;
            if (newTransactionWeight <= weight)
            {
                newTransaction.Weight += transaction.Weight;
                Transactions.Remove(transaction);
            }
            else
            {
                var overshoot = newTransactionWeight - weight;
                transaction.Weight -= overshoot;
                newTransaction.Weight += transaction.Weight;
                Transactions.Update(transaction);
            }
        }

        await SaveChangesAsync();
        return newTransaction;
    }

    public double SupplyWeight()
    {
        return Transactions.Sum(p => p.Weight);
    }
}
